package com.junmp.jyzb.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.junmp.jyzb.api.bean.dto.InventorySumDto;
import com.junmp.jyzb.api.bean.dto.WarehouseDto;
import com.junmp.jyzb.api.bean.query.InventoryReq;
import com.junmp.jyzb.api.bean.query.InventorySumReq;
import com.junmp.jyzb.api.bean.req.CabinetInOutRecordsReq;
import com.junmp.jyzb.api.bean.req.RabbitMqOrderReq;
import com.junmp.jyzb.entity.*;
import com.junmp.jyzb.service.*;
import com.junmp.v2.common.util.BeanPlusUtil;
import com.junmp.v2.db.api.factory.PageFactory;
import com.junmp.v2.db.api.factory.PageResultFactory;
import com.junmp.v2.db.api.page.PageResult;
import liquibase.pro.packaged.L;
import org.springframework.stereotype.Service;
import com.junmp.jyzb.mapper.InventorySummaryMapper;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.RequestBody;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.UUID;
import java.util.stream.Collectors;

@Service
public class InventorySummaryServiceImpl  extends ServiceImpl<InventorySummaryMapper, InventorySummary> implements InventorySummaryService {

    @Resource
    private PubOrgService pubOrgService;

    @Resource
    private OrderMainService orderMainService;

    @Resource
    private InventoryService inventoryService;

    @Resource
    private InventorySummaryMapper inventorySummaryMapper;

    @Resource
    private EquipmentSizeService equipmentSizeService;

    @Resource
    private OrderDetailService orderDetailService;

    @Resource
    private WarehouseService warehouseService;

    @Resource
    private LogSummaryService logSummaryService;

    @Resource
    private LogDetailService logDetailService;

    @Resource
    private CabinetBoxService cabinetBoxService;


    //根据条件查询库存汇总信息
    @Override
    public PageResult<InventorySummary> getEquipmentInfo(InventorySumReq req) {
        //当没有参数传递的时候默认查询全部，但是如果传递了条件，根据条件查询并且如果是根据id进行查询，则需要判断该id是否存在
        //判断组织机构是否存在
        if (ObjectUtil.isNotNull(req.getOrgId())){
            pubOrgService.PubOrgExist(req.getOrgId());
        }
        LambdaQueryWrapper<InventorySummary> wp = createWrapper(req);//取出仓库中总数并计算
        long size = list(wp).size();
        Page<InventorySummary> sumList= this.page(PageFactory.getDefaultPage(req.getPageNo(),req.getPageSize()),wp);
        List<InventorySummary> records = sumList.getRecords();

        List<InventorySummary> collect = records.stream().map(data -> {
            Inventory inventory = inventoryService.list(new LambdaQueryWrapper<Inventory>()
                    .eq(Inventory::getOrgId, data.getOrgId())
                    .eq(Inventory::getSizeId, data.getSizeId())
                    .eq(Inventory::getTypeId, data.getTypeId())).get(0);
            data.setWarrantyPeriod(inventory.getWarrantyPeriod());
            data.setMaintenancePeriod(inventory.getMaintenancePeriod());
            String photo = equipmentSizeService.getOne(new LambdaQueryWrapper<EquipmentSize>().eq(EquipmentSize::getId,inventory.getSizeId())).getPhoto();
            data.setPhoto(photo);
            return data;
        }).collect(Collectors.toList());
        Page<InventorySummary> page1 = PageFactory.getDefaultPage(req.getPageNo(), req.getPageSize());
        page1.setRecords(collect);
        page1.setTotal(size);
        return PageResultFactory.createPageResult(page1);
    }

    /**
     * 用于出库单中的装备选择
     * @return
     */
    @Override
    public List<InventorySumDto> GetInventorySumInfo(InventorySumReq req) {

        return inventorySummaryMapper.GetInventorySumInfo(req);
    }

    //处理本地主机返回单据结果
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean processInventoryRecords(RabbitMqOrderReq req) {
        //将主单据信息复制进行更新
        OrderMain orderMain = new OrderMain();
        BeanPlusUtil.copyProperties(req.getOrderMainReq(),orderMain);
        orderMainService.updateById(orderMain);

        //将子单据信息复制进行更新
        OrderDetail orderDetail = new OrderDetail();
        BeanPlusUtil.copyProperties(req.getOrderDetailReq(),orderDetail);
        orderDetailService.updateById(orderDetail);

        ////对装备表进行更新或者插入
        //新增列表
        List<Inventory> addList = new ArrayList<>();
        //更新列表
        List<Inventory> updateList = new ArrayList<>();
        String bussinessState=orderMain.getOrderState();
        //组织机构
        Long orgId;
        if (bussinessState.equals("in")){
            orgId=orderMain.getEndOrgId();
        }else {
            orgId=orderMain.getStartOrgId();
        }
        //将具体Epc进行处理（判断装备是否具有id，如果没有id表示新增一条数据在数据库中--采购）
        req.getInventoryReqList().forEach(inventoryReq -> {
            Inventory inventory = new Inventory();
            BeanPlusUtil.copyProperties(inventoryReq,inventory);
            inventory.setOrgId(orgId);
            //新增装备
            if (ObjectUtil.isNotNull(inventoryReq.getId())){
                //修改状态
                inventory.setId(UUID.randomUUID().toString());
                inventory.setLocationState(orderMain.getOrderType());
                inventory.setBussinessState("normal");
                addList.add(inventory);
            }else {
                //修改状态
                inventory.setLocationState(orderMain.getOrderType());
                //修改业务状态
                if (bussinessState.equals("in")){
                    inventory.setBussinessState("normal");
                }else {
                    inventory.setBussinessState(orderMain.getBussinessType());
                }
                updateList.add(inventory);
            }
        });
        if (ObjectUtil.isNotNull(addList) && addList.size()>0){
            inventoryService.saveBatch(addList);
        }
        if (ObjectUtil.isNotNull(updateList) && updateList.size()>0){
            inventoryService.updateBatchById(updateList);
        }
        //对装备进行记录，存入log表形成出入库记录
        Boolean c = insertLog(req,orderMain,addList,updateList);
        //对summary表中数据进行重新计算装备信息汇总
        Boolean d = updateSummaryInsByOrder(orgId);

        return (c && d);
    }
    

    //对装备进行记录，存入log表形成出入库记录
    @Transactional(rollbackFor = Exception.class)
    public Boolean insertLog(RabbitMqOrderReq req,OrderMain orderMain,List<Inventory> addList,List<Inventory> updateList ){
        //插入log_summary
        LogSummary logSummary = new LogSummary();
        logSummary.setDevice(req.getDevice());
        logSummary.setDeviceType(req.getDeviceType());
        if (orderMain.getOrderType().equals("in")){
            logSummary.setOrgId(orderMain.getEndOrgId());
            logSummary.setOrgName(orderMain.getEndOrgName());
        }else {
            logSummary.setOrgId(orderMain.getStartOrgId());
            logSummary.setOrgName(orderMain.getStartOrgName());
        }
        logSummary.setOrderMainId(orderMain.getId());
        logSummary.setBussinessType(orderMain.getBussinessType());
        boolean a = logSummaryService.save(logSummary);
        //存入log_detail表中
        List<LogDetail> logDetailList=new ArrayList<>();
        //判断是哪个列表不为空，进行出入库记录
        if (addList.size()!=0 && ObjectUtil.isNotNull(addList)){
            for (Inventory inventory:addList) {
                LogDetail logDetail = new LogDetail();
                BeanPlusUtil.copyProperties(inventory,logDetail);
                logDetail.setOutInState(orderMain.getOrderType());
                logDetail.setSummaryId(logSummary.getId());
                logDetail.setOrderMainId(orderMain.getId());
                logDetailList.add(logDetail);
            }
        }
        if (updateList.size()!=0 && ObjectUtil.isNotNull(updateList)){
            for (Inventory inventory:updateList) {
                LogDetail logDetail = new LogDetail();
                BeanPlusUtil.copyProperties(inventory,logDetail);
                logDetail.setOutInState(orderMain.getOrderType());
                logDetail.setSummaryId(logSummary.getId());
                logDetail.setOrderMainId(orderMain.getId());
                logDetailList.add(logDetail);
            }
        }
        boolean b = logDetailService.saveBatch(logDetailList);
        return (a && b);
    }

    //对summary表中数据进行重新计算装备信息汇总
    public Boolean updateSummaryInsByOrder(Long orgId){
        //查询出组织机构下的所有仓库信息，并且将表中的仓库信息数据进行删除
        List<WarehouseDto> allWarehouse = warehouseService.getAllWarehouse(orgId);
        boolean a = inventorySummaryMapper.deleteByWarehouse(allWarehouse);
        //根据仓库id重新计算汇总信息
        boolean b = inventorySummaryMapper.getSumByWarehouse(allWarehouse);
        return (a && b);
    }

    //单警柜返回数据
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean cabinetInOutRecords(CabinetInOutRecordsReq req) {
        //批量修改装备信息
        List<InventoryReq> inventoryList = req.getInventoryList();
        boolean a=true;
        for (InventoryReq inventoryReq:inventoryList) {
            Inventory inventory = new Inventory();
            BeanPlusUtil.copyProperties(inventoryReq,inventory);
            boolean result = inventoryService.updateById(inventory);
            if (!result){
                a=false;
                break;
            }
        }

        //修改summary表中数据
        boolean b = updateSummaryInsByCabinet(req.getCabinetId());
        //添加日志记录、
        Boolean c = insertCabinetLog(req);
        return (a && b && c);
    }
    public boolean updateSummaryInsByCabinet(String locationId){
        //根据箱门id查询出单警柜信息
        CabinetBox cabinetBox = cabinetBoxService.getById(locationId);
        //删除inventorysummary表中该单警柜数据
        boolean a = inventorySummaryMapper.deleteByCabinet(cabinetBox.getCabinetId());
        //重新计算该单警柜下面的装备信息
        boolean b = inventorySummaryMapper.getSumByCabinet(cabinetBox.getCabinetId());
        return (a && b);
    }

    //添加单警柜出入库记录日志
    public Boolean insertCabinetLog(CabinetInOutRecordsReq req){
        LogSummary logSummary = new LogSummary();
        BeanPlusUtil.copyProperties(req,logSummary);
        boolean a = logSummaryService.save(logSummary);
        List<InventoryReq> inventoryList = req.getInventoryList();
        List<LogDetail> logDetailList=new ArrayList<>();
        for (InventoryReq inventoryReq:inventoryList) {
            LogDetail logDetail = new LogDetail();
            BeanPlusUtil.copyProperties(inventoryReq,logDetail);
            logDetail.setCabinetId(req.getCabinetId());
            logDetail.setLocationName(req.getLocationName());
            logDetail.setOutInState(inventoryReq.getLocationState());
            logDetail.setSummaryId(logSummary.getId());
            logDetailList.add(logDetail);
        }
        boolean b = logDetailService.saveBatch(logDetailList);
        return (a && b);
    }

    private LambdaQueryWrapper<InventorySummary> createWrapper(InventorySumReq req) {
        LambdaQueryWrapper<InventorySummary> wrapper = new LambdaQueryWrapper<>();
        if (ObjectUtil.isEmpty(req)) {
            return wrapper;
        }
        //根据组织机构号查询
        wrapper.eq(ObjectUtil.isNotEmpty(req.getOrgId()), InventorySummary::getOrgId, req.getOrgId());
        //根据组织机构名模糊查询
        wrapper.like(ObjectUtil.isNotEmpty(req.getOrgName()), InventorySummary::getOrgName, req.getOrgName());
        //位置id（单警柜/仓库）
        wrapper.eq(ObjectUtil.isNotEmpty(req.getLocationId()), InventorySummary::getLocationId, req.getLocationId());
        //位置信息
        wrapper.like(ObjectUtil.isNotEmpty(req.getLocationName()), InventorySummary::getLocationName, req.getLocationName());
        //装备类别名称
        wrapper.like(ObjectUtil.isNotEmpty(req.getTypeName()), InventorySummary::getTypeName, req.getTypeName());
        //根据类别
        wrapper.eq(ObjectUtil.isNotEmpty(req.getTypeId()), InventorySummary::getTypeId, req.getTypeId());
        //型号名称
        wrapper.like(ObjectUtil.isNotEmpty(req.getSizeName()), InventorySummary::getSizeName, req.getSizeName());
        //号型
        wrapper.eq(ObjectUtil.isNotEmpty(req.getSizeId()), InventorySummary::getSizeId, req.getSizeId());
        wrapper.orderByDesc(InventorySummary::getTypeId);
        wrapper.orderByDesc(InventorySummary::getSizeId);

        return wrapper;
    }
}
